
# q-expansion of newform 1734.2.a.j, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1734
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [-1], [2], [0], [4], [-2], [0], [4], [0], [10], [-8], [2], [-10], [12], [0], [6], [12], [10], [-12], [0], [-10], [8], [4], [-6], [14], [-10], [-8], [4], [10], [-2], [0], [12], [10], [4], [-10], [24], [-2], [-4], [-16], [-6], [-12], [-14], [-16], [-18], [-14], [0], [28], [16], [-4], [-26], [-26], [0], [-2], [28], [2], [-8], [-6], [16], [-30], [-6], [-12], [-26], [-12], [0], [-10], [-6], [-20], [14], [-28], [14], [-30], [-24], [-24], [6], [4], [-16], [-26], [26], [14], [26], [-4], [22], [8], [-14], [-16], [-4], [-2], [10], [30], [32], [-12], [-24], [16], [12], [-4], [0], [30], [22], [-4], [10], [12], [-34], [36], [-6], [-28], [2], [-20], [-14], [-8], [6], [24], [38], [6], [20], [-24], [-2], [28], [-40], [-6], [36], [6], [46], [-46], [20], [28], [46], [-46], [-40], [-8], [46], [52], [16], [-8], [-10], [-6], [-30], [38], [-4], [14], [22], [20], [34], [-32], [20], [-34], [0], [2], [6], [-36], [-32], [-22], [-2], [36], [48], [-28], [-8], [24], [-2], [-22], [10], [28], [-6], [-8], [-4], [-46], [-48], [40], [2], [-34], [34], [12], [-50], [-32], [-54], [-16], [54], [-28], [50], [24], [-18], [16], [12], [2], [42], [32], [38], [-22], [4], [-26], [-8], [62], [-44], [36], [-2], [-4], [6], [46], [42], [-18], [-56], [42], [-24], [22], [-30], [-36], [30], [16], [36], [6], [12], [-2], [-42], [-16], [28], [-32], [-26], [64], [18], [16], [14], [-26], [0], [66], [-56], [-12], [-58], [-42], [56], [8], [52], [-34], [-20], [-16], [-54], [-4], [-16], [30], [66], [-12], [40], [76], [28], [-56], [-50], [-2], [0], [-40], [-68], [12], [-16], [-2], [-50], [-72], [-10], [14], [-12], [-14], [-28], [18], [-6], [24], [-44], [-30], [-70], [-2], [52], [30], [-6], [-60], [6], [10], [-12], [26], [-48], [-14], [40], [-20], [-18], [74], [36], [-48], [16], [-80], [54], [-28], [-48], [-2], [-30], [24], [34], [26], [44], [58], [-28], [26], [42], [48], [-26], [4], [68], [-22], [-18], [56], [12], [68], [14], [60], [58], [-40], [-58], [-24], [-78], [30], [84], [-88], [42], [84], [-8], [-34], [18], [12], [22], [-66], [-16], [54], [-46], [-20], [84], [-24], [-78], [-22], [-38], [40], [-76], [20], [-12], [78], [78], [-10], [0], [6], [-70], [-14], [56], [46], [-36], [66], [60], [40], [34], [4], [-26], [14], [8], [22], [-70], [32], [20], [78], [40], [66], [-58], [-32], [36], [84], [-86], [-6], [-24], [22], [-12], [-28], [72], [6], [-72], [58], [12], [-24], [34], [-14], [58], [10], [-54], [48], [-18], [-4], [-64], [80], [86], [36], [64], [62], [-78], [-36], [20], [72], [54], [-32], [-22], [-92], [38], [-54], [66], [32], [-58], [-90], [80], [-50], [-14], [20], [92], [46], [-10], [76], [44], [42], [-22], [24], [-80], [78], [-56], [14], [-14], [-40], [-4], [-26], [-18], [-36], [54], [-52], [-16]]
